@@ -15,10 +15,18 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 
-__version__ = '0.27'
+__version__ = '1.20120630'
 
 
 from settings import Settings
+from runcmd import runcmd, runcmd_unchecked
 from app import Application, AppException
 
+# The plugin system
+from hook import Hook, FilterHook
+from hookmgr import HookManager
+from plugin import Plugin
+from pluginmgr import PluginManager
+
+
 __all__ = locals()
