@@ -16,13 +16,12 @@
 
 
 import errno
-import fcntl
+import gc
 import inspect
 import logging
 import logging.handlers
 import os
-import select
-import subprocess
+import StringIO
 import sys
 import traceback
 
@@ -38,6 +37,29 @@ class AppException(Exception):
     stack trace to be written to stderr.
     
     '''
+    
+    def __init__(self, msg):
+        self.msg = msg
+        
+    def __str__(self):
+        return self.msg
+    
+    
+class LogHandler(logging.handlers.RotatingFileHandler): # pragma: no cover
+
+    '''Like RotatingFileHandler, but set permissions of new files.'''
+    
+    def __init__(self, filename, perms=0600, *args, **kwargs):
+        self._perms = perms
+        logging.handlers.RotatingFileHandler.__init__(self, filename, 
+                                                      *args, **kwargs)
+
+    def _open(self):
+        if not os.path.exists(self.baseFilename):
+            flags = os.O_CREAT | os.O_WRONLY
+            fd = os.open(self.baseFilename, flags, self._perms)
+            os.close(fd)
+        return logging.handlers.RotatingFileHandler._open(self)
 
 
 class Application(object):
@@ -84,6 +106,8 @@ class Application(object):
                                         usage=self._format_usage,
                                         description=self._format_description,
                                         epilog=epilog)
+
+        self.plugin_subdir = 'plugins'
         
     def add_settings(self):
         '''Add application specific settings.'''
@@ -120,35 +144,37 @@ class Application(object):
     def _run(self, args=None, stderr=sys.stderr, log=logging.critical):
         try:
             self.add_settings()
+            self.setup_plugin_manager()
             
             # A little bit of trickery here to make --no-default-configs and
             # --config=foo work right: we first parse the command line once,
             # and pick up any config files. Then we read configs. Finally,
             # we re-parse the command line to allow any options to override
             # config file settings.
+            self.setup()
+            self.enable_plugins()
             args = sys.argv[1:] if args is None else args
             self.parse_args(args, configs_only=True)
             self.settings.load_configs()
             args = self.parse_args(args)
 
             self.setup_logging()
-            logging.info('%s version %s starts' % 
-                         (self.settings.progname, self.settings.version))
+            self.log_config()
             
             if self.settings['output']:
                 self.output = open(self.settings['output'], 'w')
             else:
                 self.output = sys.stdout
 
-            self.setup()
             self.process_args(args)
             self.cleanup()
+            self.disable_plugins()
         except AppException, e:
             log(traceback.format_exc())
             stderr.write('ERROR: %s\n' % str(e))
             sys.exit(1)
         except SystemExit, e:
-            sys.exit(e.code)
+            sys.exit(e.code if type(e.code) == int else 1)
         except KeyboardInterrupt, e:
             sys.exit(255)
         except IOError, e: # pragma: no cover
@@ -279,9 +305,10 @@ class Application(object):
 
         if self.settings['log'] == 'syslog':
             handler = logging.handlers.SysLogHandler(address='/dev/log')
-        elif self.settings['log']:
-            handler = logging.handlers.RotatingFileHandler(
+        elif self.settings['log'] and self.settings['log'] != 'none':
+            handler = LogHandler(
                             self.settings['log'],
+                            perms=int(self.settings['log-mode'], 8),
                             maxBytes=self.settings['log-max'], 
                             backupCount=self.settings['log-keep'],
                             delay=False)
@@ -299,6 +326,47 @@ class Application(object):
         logger.addHandler(handler)
         logger.setLevel(level)
 
+    def log_config(self):
+        logging.info('%s version %s starts' % 
+                     (self.settings.progname, self.settings.version))
+        logging.debug('sys.argv: %s' % sys.argv)
+        logging.debug('environment variables:')
+        for name in os.environ:
+            logging.debug('environment: %s=%s' % (name, os.environ[name]))
+        cp = self.settings.as_cp()
+        f = StringIO.StringIO()
+        cp.write(f)
+        logging.debug('Config:\n%s' % f.getvalue())
+
+    def app_directory(self):
+        '''Return the directory where the application class is defined.
+        
+        Plugins are searched relative to this directory, in the subdirectory
+        specified by self.plugin_subdir.
+        
+        '''
+        
+        module_name = self.__class__.__module__
+        module = sys.modules[module_name]
+        dirname = os.path.dirname(module.__file__) or '.'
+        return dirname
+
+    def setup_plugin_manager(self):
+        '''Create a plugin manager.'''
+        self.pluginmgr = cliapp.PluginManager()
+        dirname = os.path.join(self.app_directory(), self.plugin_subdir)
+        self.pluginmgr.locations = [dirname]
+
+    def enable_plugins(self): # pragma: no cover
+        '''Load plugins.'''
+        for plugin in self.pluginmgr.plugins:
+            plugin.app = self
+            plugin.setup()
+        self.pluginmgr.enable_plugins()
+
+    def disable_plugins(self):
+        self.pluginmgr.disable_plugins()
+
     def parse_args(self, args, configs_only=False):
         '''Parse the command line.
         
@@ -412,174 +480,11 @@ class Application(object):
         
         '''
         
-    def runcmd(self, argv, *args, **kwargs):
-        '''Run external command or pipeline.
-
-        Example: ``runcmd(['grep', 'foo'], ['wc', '-l'], 
-                          feed_stdin='foo\nbar\n')``
-
-        Return the standard output of the command.
+    def runcmd(self, *args, **kwargs): # pragma: no cover
+        return cliapp.runcmd(*args, **kwargs)
         
-        Raise ``cliapp.AppException`` if external command returns
-        non-zero exit code. ``*args`` and ``**kwargs`` are passed
-        onto ``subprocess.Popen``.
-        
-        '''
-
-        if 'ignore_fail' in kwargs:
-            ignore_fail = kwargs['ignore_fail']
-            del kwargs['ignore_fail']
-        else:
-            ignore_fail = False
-
-        exit, out, err = self.runcmd_unchecked(argv, *args, **kwargs)
-        if exit != 0:
-            msg = 'Command failed: %s\n%s' % (' '.join(argv), err)
-            if ignore_fail:
-                logging.info(msg)
-            else:
-                logging.error(msg)
-                raise cliapp.AppException(msg)
-        return out
-        
-    def runcmd_unchecked(self, argv, *argvs, **kwargs):
-        '''Run external command or pipeline.
-
-        Return the exit code, and contents of standard output and error
-        of the command.
-        
-        See also ``runcmd``.
-        
-        '''
-
-        argvs = [argv] + list(argvs)
-        logging.debug('run external command: %s' % repr(argvs))
-
-        def pop_kwarg(name, default):
-            if name in kwargs:
-                value = kwargs[name]
-                del kwargs[name]
-                return value
-            else:
-                return default
-
-        feed_stdin = pop_kwarg('feed_stdin', '')
-        pipe_stdin = pop_kwarg('stdin', subprocess.PIPE)
-        pipe_stdout = pop_kwarg('stdout', subprocess.PIPE)
-        pipe_stderr = pop_kwarg('stderr', subprocess.PIPE)
-
-        try:
-            pipeline = self._build_pipeline(argvs,
-                                            pipe_stdin,
-                                            pipe_stdout,
-                                            pipe_stderr,
-                                            kwargs)
-            return self._run_pipeline(pipeline, feed_stdin, pipe_stdin,
-                                      pipe_stdout, pipe_stderr)
-        except OSError, e: # pragma: no cover
-            if e.errno == errno.ENOENT and e.filename is None:
-                e.filename = argv[0]
-                raise e
-            else:
-                raise
-
-    def _build_pipeline(self, argvs, pipe_stdin, pipe_stdout, pipe_stderr,
-                        kwargs):
-        procs = []
-        for i, argv in enumerate(argvs):
-            if i == 0 and i == len(argvs) - 1:
-                stdin = pipe_stdin
-                stdout = pipe_stdout
-                stderr = pipe_stderr
-            elif i == 0:
-                stdin = pipe_stdin
-                stdout = subprocess.PIPE
-                stderr = pipe_stderr
-            elif i == len(argv) - 1:
-                stdin = procs[-1].stdout
-                stdout = pipe_stdout
-                stderr = pipe_stderr
-            else:
-                stdin = procs[-1].stdout
-                stdout = subprocess.PIPE
-                stderr = pipe_stderr
-            p = subprocess.Popen(argv, stdin=stdin, stdout=stdout, 
-                                 stderr=stderr, close_fds=True, **kwargs)
-            procs.append(p)
-
-        return procs
-
-    def _run_pipeline(self, procs, feed_stdin, pipe_stdin, pipe_stdout,
-                      pipe_stderr):
-        stdout_eof = False
-        stderr_eof = False
-        out = []
-        err = []
-        pos = 0
-        io_size = 1024
-        
-        def set_nonblocking(fd):
-            flags = fcntl.fcntl(fd, fcntl.F_GETFL, 0)
-            flags = flags | os.O_NONBLOCK
-            fcntl.fcntl(fd, fcntl.F_SETFL, flags)
-
-        if feed_stdin and pipe_stdin == subprocess.PIPE:
-            set_nonblocking(procs[0].stdin.fileno())
-        if pipe_stdout == subprocess.PIPE:
-            set_nonblocking(procs[-1].stdout.fileno())
-        if pipe_stderr == subprocess.PIPE:
-            set_nonblocking(procs[-1].stderr.fileno())
-        
-        def still_running():
-            for p in procs:
-                p.poll()
-            for p in procs:
-                if p.returncode is None:
-                    return True
-            if pipe_stdout == subprocess.PIPE and not stdout_eof:
-                return True
-            if pipe_stderr == subprocess.PIPE and not stderr_eof:
-                return True # pragma: no cover
-            return False
-
-        while still_running():
-            rlist = []
-            if not stdout_eof and pipe_stdout == subprocess.PIPE:
-                rlist.append(procs[-1].stdout)
-            if not stderr_eof and pipe_stderr == subprocess.PIPE:
-                rlist.append(procs[-1].stderr)
-            
-            wlist = []
-            if pipe_stdin == subprocess.PIPE and pos < len(feed_stdin):
-                wlist.append(procs[0].stdin)
-
-            if rlist or wlist:
-                r, w, x = select.select(rlist, wlist, [])
-            else:
-                r = w = [] # pragma: no cover
-
-            if procs[0].stdin in w and pos < len(feed_stdin):
-                data = feed_stdin[pos : pos+io_size]
-                procs[0].stdin.write(data)
-                pos += len(data)
-                if pos >= len(feed_stdin):
-                    procs[0].stdin.close()
-
-            if procs[-1].stdout in r:
-                data = procs[-1].stdout.read(io_size)
-                if data:
-                    out.append(data)
-                else:
-                    stdout_eof = True
-
-            if procs[-1].stderr in r:
-                data = procs[-1].stderr.read(io_size)
-                if data:
-                    err.append(data)
-                else:
-                    stderr_eof = True
-
-        return procs[-1].returncode, ''.join(out), ''.join(err)
+    def runcmd_unchecked(self, *args, **kwargs): # pragma: no cover
+        return cliapp.runcmd_unchecked(*args, **kwargs)
 
     def _vmrss(self): # pragma: no cover
         '''Return current resident memory use, in KiB.'''
