@@ -1,4 +1,4 @@
-# Copyright (C) 2011  Lars Wirzenius
+# Copyright (C) 2009-2012  Lars Wirzenius
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,11 +31,12 @@ class Setting(object):
     nargs = 1
     choices = None
 
-    def __init__(self, names, default, help, metavar=None):
+    def __init__(self, names, default, help, metavar=None, group=None):
         self.names = names
         self.set_value(default)
         self.help = help
         self.metavar = metavar or self.default_metavar()
+        self.group = group
 
     def default_metavar(self):
         return None
@@ -74,8 +75,8 @@ class StringListSetting(Setting):
 
     action = 'append'
     
-    def __init__(self, names, default, help, metavar=None):
-        Setting.__init__(self, names, [], help, metavar=metavar)
+    def __init__(self, names, default, help, metavar=None, group=None):
+        Setting.__init__(self, names, [], help, metavar=metavar, group=group)
         self.default = default
         self.using_default_value = True
 
@@ -106,8 +107,9 @@ class ChoiceSetting(Setting):
 
     type = 'choice'
     
-    def __init__(self, names, choices, help, metavar=None):
-        Setting.__init__(self, names, choices[0], help, metavar=metavar)
+    def __init__(self, names, choices, help, metavar=None, group=None):
+        Setting.__init__(self, names, choices[0], help, metavar=metavar,
+                         group=group)
         self.choices = choices
 
     def default_metavar(self):
@@ -274,7 +276,8 @@ class Settings(object):
 
         self.string(['log'], 
                     'write log entries to FILE (default is to not write log '
-                        'files at all); use "syslog" to log to system log',
+                        'files at all); use "syslog" to log to system log, '
+                        'or "none" to disable logging',
                     metavar='FILE')
         self.choice(['log-level'], 
                     ['debug', 'info', 'warning', 'error', 'critical', 'fatal'],
@@ -287,6 +290,10 @@ class Settings(object):
                       metavar='SIZE', default=0)
         self.integer(['log-keep'], 'keep last N logs (%default)',
                      metavar='N', default=10)
+        self.string(['log-mode'], 
+                    'set permissions of new log files to MODE (octal; '
+                        'default %default)',
+                    metavar='MODE', default='0600')
 
         self.choice(['dump-memory-profile'],
                     ['simple', 'none', 'meliae', 'heapy'],
@@ -354,6 +361,15 @@ class Settings(object):
 
     def __contains__(self, name):
         return name in self._settingses
+        
+    def __iter__(self):
+        '''Iterate over canonical settings names.'''
+        for name in self._canonical_names:
+            yield name
+
+    def keys(self):
+        '''Return canonical settings names.'''
+        return self._canonical_names[:]
 
     def require(self, name):
         '''Raise exception if setting has not been set.
@@ -473,20 +489,38 @@ class Settings(object):
             else:
                 assert setting.action == 'store'
                 setting.value = value
+
+        def add_option(obj, s):
+            option_names = self._option_names(s.names)
+            obj.add_option(*option_names, 
+                           action='callback',
+                           callback=maybe(set_value),
+                           callback_args=(s,),
+                           type=s.type,
+                           nargs=s.nargs,
+                           choices=s.choices,
+                           help=s.help,
+                           metavar=s.metavar)
         
         for name in self._canonical_names:
             s = self._settingses[name]
-            option_names = self._option_names(s.names)
-            p.add_option(*option_names,
-                         action='callback',
-                         callback=maybe(set_value),
-                         callback_args=(s,),
-                         type=s.type,
-                         nargs=s.nargs,
-                         choices=s.choices,
-                         help=s.help,
-                         metavar=s.metavar)
-            p.set_defaults(**{self._destname(name): s.value})
+            if s.group is None:
+                add_option(p, s)
+                p.set_defaults(**{self._destname(name): s.value})
+
+        groups = {}
+        for name in self._canonical_names:
+            s = self._settingses[name]
+            if s.group is not None:
+                groups[s.group] = groups.get(s.group, []) + [(name, s)]
+
+        groupnames = sorted(groups.keys())
+        for groupname in groupnames:
+            group = optparse.OptionGroup(p, groupname)
+            p.add_option_group(group)
+            for name, s in groups[groupname]:
+                add_option(group, s)
+                p.set_defaults(**{self._destname(name): s.value})
 
         return p
 
