@@ -1,4 +1,4 @@
-# Copyright (C) 2011  Lars Wirzenius
+# Copyright (C) 2009-2012  Lars Wirzenius
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -48,8 +48,18 @@ class SettingsTests(unittest.TestCase):
         self.assert_('log' in self.settings)
         self.assert_('log-level' in self.settings)
 
+    def test_iterates_over_canonical_settings_names(self):
+        known = ['output', 'log', 'log-level']
+        self.assertEqual(sorted(x for x in self.settings if x in known),
+                         sorted(known))
+
+    def test_keys_returns_canonical_names(self):
+        known = ['output', 'log', 'log-level']
+        self.assertEqual(sorted(x for x in self.settings.keys() if x in known),
+                         sorted(known))
+
     def test_parses_options(self):
-        self.settings.string(['foo'], 'foo help')
+        self.settings.string(['foo'], 'foo help', group='foo')
         self.settings.boolean(['bar'], 'bar help')
         self.settings.parse_args(['--foo=foovalue', '--bar'])
         self.assertEqual(self.settings['foo'], 'foovalue')
